<?php

namespace Lc\CaracoleBundle\Model\PointSale;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Lc\CaracoleBundle\Doctrine\Extension\FilterMultipleMerchantsInterface;
use Lc\CaracoleBundle\Doctrine\Extension\OrderAmountMinTrait;
use Lc\CaracoleBundle\Doctrine\Extension\OrderAmountMinInterface;
use Lc\CaracoleBundle\Model\Address\AddressInterface;
use Lc\CaracoleBundle\Model\Merchant\MerchantInterface;
use Lc\CaracoleBundle\Model\User\UserPointSaleInterface;
use Lc\SovBundle\Doctrine\Pattern\AbstractFullEntity;

/**
 * @ORM\MappedSuperclass()
 */
abstract class PointSaleModel extends AbstractFullEntity implements FilterMultipleMerchantsInterface,
                                                                    OrderAmountMinInterface
{

    use OrderAmountMinTrait;

    /**
     * @ORM\ManyToMany(targetEntity="Lc\CaracoleBundle\Model\Merchant\MerchantInterface", inversedBy="pointSales")
     */
    protected $merchants;

    /**
     * @ORM\Column(type="string", length=63, nullable=true)
     */
    protected $code;

    /**
     * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\PointSale\PointSaleDayInfoInterface", mappedBy="pointSale", orphanRemoval=true)
     */
    protected $pointSaleDayInfos;

    /**
     * @ORM\OneToOne(targetEntity="Lc\CaracoleBundle\Model\Address\AddressInterface", inversedBy="pointSale", cascade={"persist", "remove"})
     * @ORM\JoinColumn(nullable=false)
     */
    protected $address;

    /**
     * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\User\UserPointSaleInterface", mappedBy="pointSale", orphanRemoval=true)
     */
    protected $userPointSales;


    public function __construct()
    {
        $this->merchants = new ArrayCollection();
        $this->pointSaleDayInfos = new ArrayCollection();
        $this->userPointSales = new ArrayCollection();
    }

    public function __toString()
    {
        return $this->getTitle();
    }

    public function labelAdminChoice()
    {
        return $this->getTitle();
    }

    /**
     * @return Collection|MerchantInterface[]
     */
    public function getMerchants(): Collection
    {
        return $this->merchants;
    }

    public function addMerchant(MerchantInterface $merchant): self
    {
        if (!$this->merchants->contains($merchant)) {
            $this->merchants[] = $merchant;
        }

        return $this;
    }

    public function removeMerchant(MerchantInterface $merchant): self
    {
        if ($this->merchants->contains($merchant)) {
            $this->merchants->removeElement($merchant);
        }

        return $this;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(?string $code): self
    {
        $this->code = $code;

        return $this;
    }

    /**
     * @return Collection|PointSaleDayInfoInterface[]
     */
    public function getPointSaleDayInfos(): Collection
    {
        return $this->pointSaleDayInfos;
    }

    public function addPointSaleDayInfo(PointSaleDayInfoInterface $pointSaleDayInfo): self
    {
        if (!$this->pointSaleDayInfos->contains($pointSaleDayInfo)) {
            $this->pointSaleDayInfos[] = $pointSaleDayInfo;
            $pointSaleDayInfo->setPointSale($this);
        }

        return $this;
    }

    public function removePointSaleDayInfo(PointSaleDayInfoInterface $pointSaleDayInfo): self
    {
        if ($this->pointSaleDayInfos->contains($pointSaleDayInfo)) {
            $this->pointSaleDayInfos->removeElement($pointSaleDayInfo);
            // set the owning side to null (unless already changed)
            if ($pointSaleDayInfo->getPointSale() === $this) {
                $pointSaleDayInfo->setPointSale(null);
            }
        }

        return $this;
    }

    public function getAddress(): ?AddressInterface
    {
        return $this->address;
    }

    public function setAddress(AddressInterface $address): self
    {
        $this->address = $address;

        return $this;
    }

    /**
     * @return Collection|UserPointSaleInterface[]
     */
    public function getUserPointSales(): Collection
    {
        return $this->userPointSales;
    }

    public function addUserPointSale(UserPointSaleInterface $userPointSale): self
    {
        if (!$this->userPointSales->contains($userPointSale)) {
            $this->userPointSales[] = $userPointSale;
            $userPointSale->setPointSale($this);
        }

        return $this;
    }

    public function removeUserPointSale(UserPointSaleInterface $userPointSale): self
    {
        if ($this->userPointSales->contains($userPointSale)) {
            $this->userPointSales->removeElement($userPointSale);
            // set the owning side to null (unless already changed)
            if ($userPointSale->getPointSale() === $this) {
                $userPointSale->setPointSale(null);
            }
        }

        return $this;
    }
}
